<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: DAO.php,v 1.3 2007-07-24 12:48:13 mike Exp $
 */

/**
 * Overloading of DB_DataObject (disabled by default)
 */
if (!defined('DB_DATAOBJECT_NO_OVERLOAD')) {
    define('DB_DATAOBJECT_NO_OVERLOAD', true);
}

/**
 * Requires DB_DataObject
 */
require_once 'DB/DataObject.php';

/**
 * Prefetch first result
 */
define('DAO_PREFETCH',  1);

/**
 * Calculate Found Rows
 */
define('DAO_CALC',      2);

/**
 * Cache Query
 */
define('DAO_CACHE',     4);

/**
 * Select DISTINCT
 */
define('DAO_DISTINCT',  8);

/**
 * Gadget_DAO
 * 
 * Add some convinience methods to the original DB_DataObject.
 *
 * @package     Gadget
 * @since       1.0
 */
class Gadget_DAO extends DB_DataObject
{
    /**
     * Set Select
     * 
     * @access  public
     * @return  void
     * @param   string
     */
    function selectSet($select)
    {
        $this->selectAdd();
        $this->selectAdd($select);
    }
    
    /**
     * Set Where
     * 
     * @access  public
     * @return  void
     * @param   string
     */
    function whereSet($where)
    {
        $this->whereAdd();
        $this->whereAdd($where);
    }
    
    /**
     * Quote Smart
     * 
     * @access  public
     * @return  mixed
     * @param   mixed
     */
    function quoteSmart($mixed)
    {
        $db = &$this->getDatabaseConnection();
        return $db->quoteSmart($mixed);
    }
    
    /**
     * Found Rows
     * 
     * Retrieve found rows - available with calcRows() selects.
     * 
     * @access  public
     * @return  int
     */
    function foundRows()
    {
        $db = &$this->getDatabaseConnection();
        return $db->getOne('SELECT FOUND_ROWS()');
    }
    
    /**
     * Call a method of the underlying DB object
     * 
     * @access  public
     * @return  mixed
     * @param   string
     * @param   array
     */
    function callDB($method, $params = array())
    {
        $db = &$this->getDatabaseConnection();
        return call_user_func_array(array(&$db, $method), $params);
    }
    
    /**
     * Replace
     * 
     * @access  public
     * @return  bool
     */
    function replace($whereadd_only = true)
    {
        $updated = $this->update($whereadd_only);
        return $updated ? true : $this->insert();
    }
    
    /**
     * To Object
     * 
     * @access  public
     * @return  object
     */
    function toObject()
    {
        return (object) $this->toArray();
    }
    
    /**
     * Now
     * 
     * @access  public
     * @return  string
     */
    function now()
    {
        return date('Y-m-d H:i:s');
    }
    
    /**
     * Datetime
     * 
     * @access  public
     * @return  string
     * @param   int     $diff
     * @param   bool    $full
     */
    function datetime($diff = 0, $full = true)
    {
        return '"'. date($full ? 'Y-m-d H:i:s' : 'Y-m-d', TIME + $diff) .'"';
    }
    
    /**
     * Timestamp
     * 
     * @access  public
     * @return  string
     * @param   int     $diff
     */
    function timestamp($diff = 0, $length = 14, $fill = false)
    {
        return '"'. substr(date('YmdHis', TIME + $diff), 0, $length) . ($fill ? str_repeat('0', 14 - $length):'') .'"';
    }
    
    /**
     * Update
     * 
     * @access  public
     * @return  mixed
     */
    function update($whereAddOnly = false)
    {
        $this->modified = $this->now();
        return parent::update($whereAddOnly);
    }
    
    /**
     * Insert
     * 
     * @access  public
     * @return  mixed
     */
    function insert()
    {
        $this->modified = 0;
        $this->created = $this->now();
        return parent::insert();
    }
    
    /**
     * Find
     * 
     * @access  public
     * @return  mixed
     */
    function find($mask = 0)
    {
        $prefetch = false;
        if ($mask === true) {
            $prefetch = true;
        } else {
            if ($mask & DAO_PREFETCH) {
                $prefetch = true;
            }
            if ($mask & DAO_CALC) {
                $this->_query['data_select'] = 'SQL_CALC_FOUND_ROWS '. 
                $this->_query['data_select'];
            }
            if ($mask & DAO_CACHE) {
                $this->_query['data_select'] = 'SQL_CACHE '. 
                $this->_query['data_select'];
            }
            if ($mask & DAO_DISTINCT) {
                $this->_query['data_select'] = 'DISTINCT '.
                $this->_query['data_select'];
            }
        }
        return parent::find($prefetch);
    }
    
    /**
     * Query
     * 
     * @access  private (sort of...)
     * @return  mixed
     * @param   string  $query
     */
    function _query($query)
    {
        if (Gadget::getConf('Log', 'enabled') && Gadget::getConf('Log', 'query')) {
            $tstamp = array_sum(explode(' ', microtime()));
            $result = parent::_query($query);
            $tstamp = array_sum(explode(' ', microtime())) - $tstamp;
            Gadget::log('QUERY', sprintf('[%-20s] (%0.6f) %s', get_class($this), $tstamp, $query));
            return $result;
        }
        return parent::_query($query);
    }
    
    /**
     * Get Select Query
     * 
     * @access  public
     * @return  string
     */
    function getSelectQuery()
    {
        $dao = clone($this);
        $dao->_build_condition($dao->table());
        
        if (empty($GLOBALS['_DB_DATAOBJECT']['CONFIG']['quote_identifiers'])) {
            $table = $dao->__table;
        } else {
            $table = $this->callDb('quoteIdentifier', array($dao->__table));
        }
        
        $query = 'SELECT '.
            $dao->_query['data_select'] .
            ' FROM '. $table .' ' .
            $dao->_join .
            $dao->_query['condition'] . ' '.
            $dao->_query['group_by']  . ' '.
            $dao->_query['having']    . ' '.
            $dao->_query['order_by'];
        if (    isset($dao->_query['limit_start']) && 
                strlen($dao->_query['limit_start'] . 
                    $dao->_query['limit_count'])) {
            $query = $this->callDb('modifyLimitQuery', 
                array($query, $dao->_query['limit_start'], 
                    $dao->_query['limit_count']));
        }
        return $query;
    }
    
    /**
     * Union
     * 
     * @access  public
     * @return  bool
     * @param   array   $daos
     * @param   string  $append
     */
    function union($daos, $append = '')
    {
        if (!count($daos)) {
            return false;
        }
        $unions = array();
        foreach ($daos as $dao) {
            $unions[] = $dao->getSelectQuery();
        }
        $union = '('. implode(') UNION (', $unions) .') '. $append;
        return !PEAR::isError($this->query($union));
    }
    
    /**
     * Not Modified
     * 
     * @access  public
     * @return  bool
     */
    function wasModified()
    {
        return $this->modified && $this->modified !== '0000-00-00 00:00:00';
    }
    
}

?>
