<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.10 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Tree.php,v 1.10 2007-03-31 06:01:54 mike Exp $
 */


/**
 * Track Errors
 */
ini_set('track_errors', true);

/**
 * Requires Gadget_Object
 */
require_once 'Gadget/Object.php';

/**
 * Requires Gadget_Tree_Node
 */
require_once 'Gadget/Tree/Node.php';

/**#@+
 * Error Constants
 */
define('GADGET_TREE_E_UNKOWN', 0);
define('GADGET_TREE_E_INVALID_FILE', -1);
define('GADGET_TREE_E_INVALID_PARAM', -2);
define('GADGET_TREE_E_CLASS_NOT_FOUND', -3);
define('GADGET_TREE_E_CHILD_NOT_FOUND', -4);
define('GADGET_TREE_E_INVALID_OP', -5);
define('GADGET_TREE_E_INVALID_PATH', -6);
/**#@-*/

/**
 * Gadget_Tree
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree extends Gadget_Object
{
    /**#@+
     * @access protected 
     * @type object
     */
    var $root;
    var $node;
    /**#@-*/

    /**
     * Max Depth
     * 
     * The maximum depth of the tree.  Set to 0 if the tree should only consist
     * of the root node.  Set to -1 for infinite depth (default).
     * 
     * @var int 
     */
    var $maxDepth = -1;

    /**
     * Factory
     * 
     * Gadget_Tree factory.
     * 
     * @access  public 
     * @return  object Returns Gadget_Tree on success or PEAR_Error on failure.
     * @param   string $type 
     */
    function &factory($type = null, $options = array())
    {
        $class = isset($type) ? 'Gadget_Tree_'. $type : 'Gadget_Tree';
        $instance = &Gadget_Tree::create($options, $class);
        return $instance;
    }

    /**
     * Create
     * 
     * Generic object creator which follows the PEAR directory standard.
     * Note that the class' constructor should only accept 1 parameter.
     * 
     * @static 
     * @access  public 
     * @return  object 
     * @param   mixed   $options    Class' construction parameter.
     * @param   string  $class      Class' full name, eg: Gadget_Tree_XML
     */
    function &create($options = null, $class = __CLASS__)
    {
        if (!class_exists($class, false)) {
            if (!@include_once strtr($class, '_', '/') . '.php') {
                $instance = &Gadget_Tree::raiseError(
                    "Cannot create instance of class '$class'.",
                    GADGET_TREE_E_CLASS_NOT_FOUND
                );
                return $instance;
            }
        }
        $instance = &new $class($options);
        return $instance;
    }

    /**
     * Serialize
     * 
     * @static 
     * @access  public
     * @return  string
     * @param   object  Gadget_Tree $tree
     * @param   array   $options Serializer options
     * @param   string  $serilaizer Gadget_Tree_Serializer class
     */
    function serialize(&$tree, $options = array(), $serializer = 'Gadget_Tree_Serializer')
    {
        $serializer = &Gadget_Tree::create($options, $serializer);
        if (Gadget_Tree::isError($serializer)) {
            return $serializer;
        }
        return $serializer->serializeTree($tree);
    }

    /**
     * Raise Error
     * 
     * Raises a PEAR_Error.
     * 
     * @access  protected 
     * @return  object  PEAR_Error
     * @param   string  $error The error message.
     * @param   int     $code The error code.
     */
    function &raiseError($error, $code = 0)
    {
        require_once 'PEAR.php';
        $error = &PEAR::raiseError($error, $code);
        return $error;
    }

    /**
     * Is Error
     * 
     * @access  public 
     * @return  bool 
     */
    function isError($error)
    {
        return is_a($error, 'PEAR_Error');
    }

    /**
     * Unfold Tree
     * 
     * Unfolds a Gadget_Tree object to a flat array.
     * 
     * @static
     * @access  public 
     * @return  array   Returns the flat array (1 dimension).
     * @param   object  $tree The Gadget_Tree object to unfold.
     */
    function &unfoldTree(&$tree)
    {
        Gadget_Tree::unfold($array = array(&$tree->root), $into = array());
        return $into;
    }

    /**
     * Unfold
     * 
     * Unfolds an array of nodes to a flat array.
     * 
     * @static
     * @access  public 
     * @return  void 
     * @param   array   $tree The array of Gadget_Tree_Node objects.
     * @param   array   $into The flat array of the tree nodes.
     */
    function unfold(&$tree, &$into)
    {
        for ($i = 0, $c = count($tree); $i < $c; $i++) {
            $into[] = &$tree[$i];
            if ($tree[$i]->count()) {
                Gadget_Tree::unfold($tree[$i]->childs, $into);
            }
        }
    }

    /**
     * From
     * 
     * Set the tree from an Gadget_Tree_Node object.
     * 
     * @access  protected 
     * @param   mixed   $from 
     * @param   array   $testFrom 
     */
    function &from(&$from, $testFrom = array())
    {
        if ($this->fromNode($from)) {
            return $this->root;
        }
        foreach ($testFrom as $method => $testError) {
            if ($testError) {
                if (!Gadget_Tree::isError($this->{"from$method"}($from))) {
                    return $this->root;
                }
            } elseif ($this->{"from$method"}($from)) {
                return $this->root;
            }
        }
        return false;
    }

    /**
     * From Node
     * 
     * Set the tree from an Gadget_Tree_Node object.
     * 
     * @access  public 
     * @return  mixed   Returns the root node if $node is_a() Gadget_Tree_Node.
     * @param   object  $node The root Gadget_Tree_Node object.
     */
    function &fromNode(&$node)
    {
        if (is_a($node, 'Gadget_Tree_Node')) {
            $this->root = &$node;
        }
        return $this->root;
    }

    /**
     * Dump
     * 
     * Prints the string representation of the tree.
     * 
     * @access  public 
     * @return  void 
     * @param   array   $options Serializer options
     */
    function dump($options = array())
    {
        echo $this->toString($options);
    }

    /**
     * To Array
     * 
     * @access  public 
     * @return  array 
     */
    function &toArray()
    {
        $array = &$this->root->toArray();
        return $array;
    }

    /**
     * To File
     * 
     * Save the tree to an file.  The output is specific to the tree class.
     * 
     * @access  public 
     * @return  mixed   Returns true on success or a PEAR_Error on failure.
     * @param   string  $file The path of the destination file.
     * @param   array   $options Serializer options
     */
    function toFile($file, $options = array())
    {
        return Gadget_Tree::fputs($file, $this->toString($options));
    }

    /**
     * To String
     * 
     * @access  public 
     * @return  string
     * @param   array   $options Serializer options
     */
    function toString($options = array())
    {
        return $this->serialize($this, $options);
    }

    /**
     * fputs
     * 
     * @static
     * @access  protected 
     * @return  mixed   Returns true on success or a PEAR_Error on failure.
     * @param   string  $contents File contents
     */
    function fputs($file, $contents)
    {
        if (!is_dir($dir = dirname($file))) {
            require_once 'System.php';
            if (!System::mkdir(array('-p', $dir))) {
                return Gadget_Tree::raiseError($php_errormsg, GADGET_TREE_E_INVALID_FILE);
            }
        }
        if (function_exists('file_put_contents')) {
            return @file_put_contents($file, $contents) ? true :
                Gadget_Tree::raiseError($php_errormsg, GADGET_TREE_E_INVALID_FILE);
        }
        if (!is_resource($f = @fopen($file, 'wb'))) {
            return Gadget_Tree::raiseError($php_errormsg, GADGET_TREE_E_INVALID_FILE);
        }
        @flock($f, LOCK_EX);
        if (1 > @fwrite($f, $contents)) {
            return Gadget_Tree::raiseError($php_errormsg, GADGET_TREE_E_INVALID_FILE);
        }
        @flock($f, LOCK_UN);
        @fclose($f);
        return true;
    }

    /**
     * Walk along a Link
     * 
     * @static
     * @access  public 
     * @return  array   Flat array with all nodes along the link.
     * @param   object  Gadget_Tree_Node $node The starting node.
     * @param   string  $link The literal link name.
     * @param   bool    $withFirstNode Whether the first node should be included.
     */
    function &walkLink(&$node, $link, $withFirstNode = false)
    {
        if ($withFirstNode) {
            $links = array(&$node);
        } else {
            $links = array();
        }
        while ($node = &$node->$link) {
            $links[] = &$node;
        }
        return $links;
    }

    /**
     * Search By Path
     * 
     * @todo    implement
     * @access  public 
     * @return  mixed 
     */
    function &searchByPath($path, $pathSeparator = '/')
    {
        require_once 'Gadget/Tree/Search.php';
        $search = &new Gadget_Tree_Search($path, $pathSeparator);
        $search->parse();
        $result = &$search->search($this->root);
        return $result;
    }
}

?>
